(function (Drupal, drupalSettings, once) {
  Drupal.behaviors.salesforceMFWSetup = {
    attach: (context) => {
      if (!once('salesforceMFWSetup', 'html').length) {
        return;
      }

      const salesforceMfw = drupalSettings.salesforce_mfw || {};
      if (Object.keys(salesforceMfw).length) {
        // We can only support one configuration per page due to salesforce bootstrap
        // limitation. @see module documentation for more information.
        // we need to look for the first salesforce utility in the list that has chat_visible.
        const utility = Object.values(salesforceMfw).find((value) => {
          const storageKey = `${value.configuration.organization_id}_WEB_STORAGE`;
          const salesforceJWT =
            typeof Storage !== 'undefined'
              ? JSON.parse(localStorage.getItem(storageKey))
              : false;

          const matched =
            salesforceJWT &&
            'JWT' in salesforceJWT &&
            value.configuration.allow_path_override;

          return value.configuration.chat_visible === true || matched;
        });

        if (utility) {
          const script = document.createElement('script');
          script.type = 'text/javascript';
          script.src = utility.configuration.utility_bootstrap;
          script.async = true;
          script.onload = () => {
            /*
             * Dispatch an event so that third-parties are able to
             * override configuration via Javascript event listeners.
             * Placing in the script's onload will allow consumers to
             * modify configuration after all dependencies have loaded.
             */
            window.dispatchEvent(
              new CustomEvent('SalesforceMFWConfig', {
                detail: utility.configuration,
              }),
            );

            try {
              const embeddedserviceBootstrap = embeddedservice_bootstrap || {};

              embeddedserviceBootstrap.settings.language =
                utility.configuration.language;
              window.addEventListener('onEmbeddedMessagingReady', () => {
                const prechatHiddenFields = {};
                const prechatVisibleFields = {};
                // Iterate over the fields and translate into expected format.

                if (utility.configuration.prechat_fields.length) {
                  utility.configuration.prechat_fields.forEach((field) => {
                    if (field.hidden) {
                      prechatHiddenFields[field.name] = field.default_value;
                    } else {
                      prechatVisibleFields[field.name] = {
                        value: field.default_value,
                        isEditableByEndUser: Boolean(field.editable_by_user),
                      };
                    }
                  });
                }

                // Set visible pre-chat fields
                embeddedserviceBootstrap.prechatAPI.setVisiblePrechatFields(
                  prechatVisibleFields,
                );

                // Set hidden pre-chat fields
                embeddedserviceBootstrap.prechatAPI.setHiddenPrechatFields(
                  prechatHiddenFields,
                );
              });
              embeddedserviceBootstrap.init(
                utility.configuration.organization_id,
                utility.configuration.config_name,
                utility.configuration.site_url,
                { scrt2URL: utility.configuration.snippet_config },
              );
            } catch (err) {
              console.error('Error loading Embedded Messaging: ', err);
            }
          };
          document.head.appendChild(script);
        }
      }
    },
  };
})(Drupal, drupalSettings, once);
