<?php

/**
 * @file
 * Post update functions for salesforce_mfw module.
 */

/**
 * Issue #3524814: Pre-chat fields can be saved as an object instead of array.
 */
function salesforce_mfw_post_update_resave_block_configurations() {
  $block_storage = \Drupal::entityTypeManager()->getStorage('block');
  $block_instances = $block_storage->loadByProperties([
    'plugin' => 'salesforce_mfw',
  ]);

  // Set the pre-chat fields in each block to the array values
  // or an empty array before saving.
  foreach ($block_instances as $block) {
    try {
      $config = $block->get('settings');
      $prechat_fields = $config['prechat_fields'];
      $config['prechat_fields'] = is_array($prechat_fields) ? array_values($prechat_fields) : [];
      $block->set('settings', $config);
      $block->save();
    }
    catch (\Exception $e) {
      \Drupal::logger('salesforce_mfw')->error('Failed to resave block @id: @message', [
        '@id' => $block->id(),
        '@message' => $e->getMessage(),
      ]);
    }
  }
}
